@@ -6,13 +6,14 @@ import sys
 import distutils.filelist
 import platform
 import types
+import functools
+from importlib import import_module
+import inspect
 
-from .py26compat import import_module
-from setuptools.extern import six
+import six
 
 import setuptools
 
-
 __all__ = []
 """
 Everything is private. Contact the project team
@@ -20,6 +21,20 @@ if you think you need this functionality.
 """
 
 
+def _get_mro(cls):
+    """
+    Returns the bases classes for cls sorted by the MRO.
+
+    Works around an issue on Jython where inspect.getmro will not return all
+    base classes if multiple classes share the same name. Instead, this
+    function will return a tuple containing the class itself, and the contents
+    of cls.__bases__. See https://github.com/pypa/setuptools/issues/1024.
+    """
+    if platform.python_implementation() == "Jython":
+        return (cls,) + cls.__bases__
+    return inspect.getmro(cls)
+
+
 def get_unpatched(item):
     lookup = (
         get_unpatched_class if isinstance(item, six.class_types) else
@@ -35,25 +50,23 @@ def get_unpatched_class(cls):
     Also ensures that no other distutils extension monkeypatched the distutils
     first.
     """
-    while cls.__module__.startswith('setuptools'):
-        cls, = cls.__bases__
-    if not cls.__module__.startswith('distutils'):
+    external_bases = (
+        cls
+        for cls in _get_mro(cls)
+        if not cls.__module__.startswith('setuptools')
+    )
+    base = next(external_bases)
+    if not base.__module__.startswith('distutils'):
         msg = "distutils has already been patched by %r" % cls
         raise AssertionError(msg)
-    return cls
+    return base
 
 
 def patch_all():
     # we can't patch distutils.cmd, alas
     distutils.core.Command = setuptools.Command
 
-    has_issue_12885 = (
-        sys.version_info < (3, 4, 6)
-        or
-        (3, 5) < sys.version_info <= (3, 5, 3)
-        or
-        (3, 6) < sys.version_info
-    )
+    has_issue_12885 = sys.version_info <= (3, 5, 3)
 
     if has_issue_12885:
         # fix findall bug in distutils (http://bugs.python.org/issue12885)
@@ -62,13 +75,9 @@ def patch_all():
     needs_warehouse = (
         sys.version_info < (2, 7, 13)
         or
-        (3, 0) < sys.version_info < (3, 3, 7)
-        or
         (3, 4) < sys.version_info < (3, 4, 6)
         or
         (3, 5) < sys.version_info <= (3, 5, 3)
-        or
-        (3, 6) < sys.version_info
     )
 
     if needs_warehouse:
@@ -76,7 +85,6 @@ def patch_all():
         distutils.config.PyPIRCCommand.DEFAULT_REPOSITORY = warehouse
 
     _patch_distribution_metadata_write_pkg_file()
-    _patch_distribution_metadata_write_pkg_info()
 
     # Install Distribution throughout the distutils
     for module in distutils.dist, distutils.core, distutils.cmd:
@@ -100,31 +108,21 @@ def _patch_distribution_metadata_write_pkg_file():
     )
 
 
-def _patch_distribution_metadata_write_pkg_info():
+def patch_func(replacement, target_mod, func_name):
     """
-    Workaround issue #197 - Python 3 prior to 3.2.2 uses an environment-local
-    encoding to save the pkg_info. Monkey-patch its write_pkg_info method to
-    correct this undesirable behavior.
+    Patch func_name in target_mod with replacement
+
+    Important - original must be resolved by name to avoid
+    patching an already patched function.
     """
-    environment_local = (3,) <= sys.version_info[:3] < (3, 2, 2)
-    if not environment_local:
-        return
+    original = getattr(target_mod, func_name)
 
-    distutils.dist.DistributionMetadata.write_pkg_info = (
-        setuptools.dist.write_pkg_info
-    )
-
-
-def patch_func(replacement, original):
-    # first set the 'unpatched' attribute on the replacement to
+    # set the 'unpatched' attribute on the replacement to
     # point to the original.
     vars(replacement).setdefault('unpatched', original)
 
-    # next resolve the module in which the original func resides
-    target_mod = import_module(original.__module__)
-
-    # finally replace the function in the original module
-    setattr(target_mod, original.__name__, replacement)
+    # replace the function in the original module
+    setattr(target_mod, func_name, replacement)
 
 
 def get_unpatched_function(candidate):
@@ -139,37 +137,43 @@ def patch_for_msvc_specialized_compiler():
     # import late to avoid circular imports on Python < 3.5
     msvc = import_module('setuptools.msvc')
 
-    try:
-        # Distutil file for MSVC++ 9.0 and upper (Python 2.7 to 3.4)
-        import distutils.msvc9compiler as msvc9compiler
-    except ImportError:
-        pass
-
-    try:
-        # Distutil file for MSVC++ 14.0 and upper (Python 3.5+)
-        import distutils._msvccompiler as msvc14compiler
-    except ImportError:
-        pass
-
     if platform.system() != 'Windows':
         # Compilers only availables on Microsoft Windows
         return
 
+    def patch_params(mod_name, func_name):
+        """
+        Prepare the parameters for patch_func to patch indicated function.
+        """
+        repl_prefix = 'msvc9_' if 'msvc9' in mod_name else 'msvc14_'
+        repl_name = repl_prefix + func_name.lstrip('_')
+        repl = getattr(msvc, repl_name)
+        mod = import_module(mod_name)
+        if not hasattr(mod, func_name):
+            raise ImportError(func_name)
+        return repl, mod, func_name
+
+    # Python 2.7 to 3.4
+    msvc9 = functools.partial(patch_params, 'distutils.msvc9compiler')
+
+    # Python 3.5+
+    msvc14 = functools.partial(patch_params, 'distutils._msvccompiler')
+
     try:
         # Patch distutils.msvc9compiler
-        patch_func(msvc.msvc9_find_vcvarsall, msvc9compiler.find_vcvarsall)
-        patch_func(msvc.msvc9_query_vcvarsall, msvc9compiler.query_vcvarsall)
-    except NameError:
+        patch_func(*msvc9('find_vcvarsall'))
+        patch_func(*msvc9('query_vcvarsall'))
+    except ImportError:
         pass
 
     try:
         # Patch distutils._msvccompiler._get_vc_env
-        patch_func(msvc.msvc14_get_vc_env, msvc14compiler._get_vc_env)
-    except NameError:
+        patch_func(*msvc14('_get_vc_env'))
+    except ImportError:
         pass
 
     try:
         # Patch distutils._msvccompiler.gen_lib_options for Numpy
-        patch_func(msvc.msvc14_gen_lib_options, msvc14compiler.gen_lib_options)
-    except NameError:
+        patch_func(*msvc14('gen_lib_options'))
+    except ImportError:
         pass
